package cn.com.qmth.examcloud.core.examwork.api.controller;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.api.commons.enums.DataRuleType;
import cn.com.qmth.examcloud.api.commons.exchange.PageInfo;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.api.commons.security.bean.UserDataRule;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.PathUtil;
import cn.com.qmth.examcloud.core.examwork.base.util.ExportUtils;
import cn.com.qmth.examcloud.core.examwork.service.OrgIpService;
import cn.com.qmth.examcloud.core.examwork.service.bean.orgip.OrgIpInfo;
import cn.com.qmth.examcloud.core.examwork.service.bean.orgip.OrgIpQuery;
import cn.com.qmth.examcloud.core.examwork.service.bean.orgip.OrgIpSave;
import cn.com.qmth.examcloud.web.security.DataRule;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "学习中心IP接口")
@RequestMapping("${$rmp.ctr.examwork}/org/ip")
public class OrgIpController extends ControllerSupport {

	@Autowired
	private OrgIpService orgIpService;

	@DataRule(type = { DataRuleType.ORG })
	@ApiOperation(value = "分页查询")
	@PostMapping("page")
	public PageInfo<OrgIpInfo> page(OrgIpQuery req) {
		User user = getAccessUser();
		req.setRootOrgId(user.getRootOrgId());
		UserDataRule orgUd = getUserDataRule(DataRuleType.ORG);
		req.setOrgUd(orgUd);
		return orgIpService.getPage(req);
	}

	@ApiOperation(value = "新增")
	@PostMapping("save")
	public void add(OrgIpSave req) {
		User user = getAccessUser();
		req.setUser(user);
		orgIpService.add(req);
	}

	@ApiOperation(value = "修改")
	@PostMapping("update")
	public void update(OrgIpSave req) {
		User user = getAccessUser();
		req.setUser(user);
		orgIpService.update(req);
	}

	@ApiOperation(value = "删除")
	@PostMapping("delete")
	public void delete(@RequestBody ArrayList<Long> ids) {
		User user = getAccessUser();
		orgIpService.delete(ids, user.getRootOrgId());
	}

	@ApiOperation(value = "下载导入模板")
	@GetMapping("template")
	public void template(HttpServletResponse response) {
		String resoucePath = PathUtil.getResoucePath("templates/orgIpImportTemplate.xlsx");
		exportFile("IP登记导入模板.xlsx", new File(resoucePath));
	}

	@DataRule(type = { DataRuleType.ORG })
	@ApiOperation(value = "导入IP")
	@PostMapping("import")
	public Map<String, Object> importIp(@RequestParam CommonsMultipartFile file) {
		UserDataRule orgUd = getUserDataRule(DataRuleType.ORG);
		if (orgUd.assertEmptyQueryResult()) {
			throw new StatusException("学习中心权限为空");
		}
		List<String> failRecords = orgIpService.importIp(getAccessUser(), orgUd, file);
		Map<String, Object> map = Maps.newHashMap();
		map.put("hasError", CollectionUtils.isNotEmpty(failRecords));
		map.put("failRecords", failRecords);
		return map;
	}

	@DataRule(type = { DataRuleType.ORG })
	@ApiOperation(value = "导出")
	@GetMapping("export")
	public void export(@RequestParam(required = false) Long orgId,HttpServletResponse response) {
		OrgIpQuery req=new OrgIpQuery();
		req.setOrgId(orgId);
		UserDataRule orgUd = getUserDataRule(DataRuleType.ORG);
		List<OrgIpInfo> ret;
		if (orgUd.assertEmptyQueryResult()) {
			ret=new ArrayList<>();
		}else {
			User user = getAccessUser();
			req.setRootOrgId(user.getRootOrgId());
			req.setOrgUd(orgUd);
			ret=orgIpService.listForExport(req);
		}
		ExportUtils.exportEXCEL("考点IP登记", OrgIpInfo.class, ret, response);
	}
	
	@ApiOperation(value = "同步到考试")
	@PostMapping("exam/add")
	public void examAdd(@RequestParam Long examId) {
		User user = getAccessUser();
		orgIpService.examAdd(user.getRootOrgId(),examId);
	}
	
	@ApiOperation(value = "获取ip")
	@PostMapping("show")
    public String getIp(HttpServletRequest request) {
        return getIp(request);
	}
}
